<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
	xmlns:as="http://www.oasis-open.org/committees/cam" 
	xmlns:camed="http://jcam.org.uk/editor"
			xmlns:w2e="http://thisSheet" 
   			exclude-result-prefixes="xs xsd camed w2e" version="1.0">

	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
     Author: David RR Webber, February, 2008 -->
	<!--
    Purpose  : Extract XSD inlined constraint and content model definitions into CAM template rules
    Original : 03/31/2008
    Initials : drrw
    CAM ver    : 1.1 
    Description : Part of a set of tools to ease alignment of XSD based validations and documentation
                  and CAM templates.
    Updated  - Inits - Change
    02/20/08   drrw    Initial release
    02/23/08   drrw    Added parsing of XSD attribute definitions
    02/23/08   drrw    Added integer type
    02/28/08   drrw    Add additional types - date, time, nonNegativeInteger, positiveInteger, gMonth, gYear, gDay et al
    03/11/08   drrw    Add logic to select choice condition (parent / *)
    03/16/08   drrw    Namespace handling
    03/17/08   drrw    path item types itemName lookup
    03/31/08   drrw    Process inlined rules from xsd example
    04/01/08   mmer  Refactored code
    04/01/08   drrw    For GetName exclude any xsd:{name} parent 
    04/04/08   drrw    Extend CAM type handling for time and date types.
    04/04/08   drrw    Add enumeration support for variables.
    04/05/08   drrw    Extend parent path 2 deep for rules generation (xsd:choice still gives issue though)
    04/06/08   drrw    Handle element with simpleType enumerations.
    04/06/08   drrw    Resolve XPath with xsd:choice for enumerations
    04/08/08   drrw    Add xsd:base type handling and default typing redefines
    04/11/08   drrw    Correct quote and string length generation in rules
    04/14/08   drrw    Fix Z quote
    04/16/08   drrw    Add duration restriction
    04/16/08   drrw    Add xsd:int type
    04/16/08   drrw    Add W3C type check and datatype() assertion
    04/17/08   drrw    Comment out non-essential debugging messages
    05/06/08   drrw    Handle resolved data types
    05/08/08   drrw    Fix for maxOccurs unbounded handling
    05/08/08   drrw    Enhance default dateTime format mask rule handling
    05/12/08   drrw    If xsd:choice is not group - do not output [ ] paren's
    05/12/08   drrw    set xslt version to 1.0 not 2.0
    05/12/08   drrw    handle choice within choice rules generation
    05/14/08   drrw    suppress makeRepeatable() when maxOccurs = '1'
    05/21/08   drrw    annotation for restrict values
    05/22/08   drrw    add datatype and xpath for xsd:any content
    05/23/08   drrw    correct generation xpaths for anyAttribute
    06/20/08   drrw    add min/maxlength support
    07/27/08   drrw    improve parent logic to remove duplicate rules collisions
    07/27/08   drrw    improve itemname path for choice items
    07/27/08   drrw    omit w2e namespace declaration
    08/02/08   drrw    handle float datatype
    08/02/08   drrw    adjust integer mask based on xsd:digits
    11/06/08   drrw    handle pattern regexp to setStringMask
    11/07/08   drrw    xsd:length on xsd:string type restriction mapping
    11/09/08   drrw    align date mask with xsd:pattern
    12/11/08   drrw    align fixed attribute with fixed structure value
    12/29/08   drrw    change conditional XPath to relative target
    01/02/09   drrw    setchoice() use = instead of contains for non-namespaced items
    01/03/09   drrw    fix outputting of extra } in regexp pattern mask
    01/11/09   drrw    enhance regexp pattern mask handling
    02/04/09   drrw    support mask for xsd decimal with zero fractionDigits kludge
    02/04/09   drrw    handle string with maxLength restriction
    02/09/09   drrw    fix attribute facet handling
    03/19/09   drrw    add pattern A-H,J-N,P-Z
    04/27/09   drrw    handle xsd:union of date types
    04/14/09   drrw    add handling of xsd:union of xsd:integer xsd:decimal types 
    04/15/09   drrw    add lookup union types
    06/30/09   drrw    Handle regex patterns with ( ) operators and UK post code mask
    07/02/09   drrw    Adjust numeric repeat count for ten multiples
    08/18/09   drrw    setChoice for complexTypes returns /// in XPath - so replace with //
    10/16/09   drrw    Handle structure annotations from dictionary build modes
    03/16/10   drrw    Vaguries of decimal for models with zero fractions
    12/02/10   drrw    Add QName to list of W3C types
    12/02/10   drrw    Handle default values on elements
    05/18/11   drrw    Fix issue with multiple setNumberRange() for xsd:integer with xsd:restriction
    05/18/11   drrw    Fix processing of numeric restriction on attribute
    06/02/11   drrw    Add w2e:nodeParentMatch function and resolve xsd:choice path issues
    06/03/11   drrw    Add handling of xsd:choice and xsd:maxOccurs logic
    06/04/11   drrw    Default use of optional for attributes
    07/10/11  drrw    Improve xsd:pattern loading
    06/01/12  drrw    Enhance xsd:pattern handling for UK postcode and BFPO regexp
    06/01/12  drrw    Enhance xsd:pattern handling for multiple patterns on one element
    06/12/12  drrw    Fix for [0-9A-Za-z'\.\-_] patterns and optional
    06/23/12  drrw    Detect xsd:pattern restriction GDSC
    10/05/12  drrw    Telephone number mask handling
    11/17/12  drrw    Fixed handling of base types without xsd: prefix 
    01/27/13  drrw    Inline xsd:base handling
    03/25/13  drrw    Change date rules to check for period instead of length
-->
	<xsl:output indent="yes"/>
	<xsl:template name="rules">
		<xsl:param name="root"/>
		<as:BusinessUseContext>
			<as:Rules>
				<as:default>
					<as:context>
						<xsl:apply-templates select="$root/*" mode="rules"/>
					</as:context>
				</as:default>
			</as:Rules>
		</as:BusinessUseContext>
	</xsl:template>

	<xsl:template match="*" mode="rules">
		<xsl:param name="choice"/>
				<!-- process each element -->
		<xsl:if test="$logLevel &gt; 3"><xsl:message>Element name <xsl:value-of select="name()"/></xsl:message></xsl:if>
		
		<xsl:variable name="this" select="name()"></xsl:variable>
		<xsl:variable name="p1" select="parent::node()/name()"/>
		<xsl:variable name="p2" select="../parent::node()/name()"/>
		<xsl:variable name="p3" select="../../parent::node()/name()"/>
		<xsl:variable name="p4" select="../../../parent::node()/name()"/>
		<xsl:variable name="p5" select="../../../../parent::node()/name()"/>
		<xsl:variable name="ancestor">
			<xsl:choose>
				<xsl:when test="$p1='xsd:choice' and($p2='xsd:choice')"><xsl:value-of select="$p3"/></xsl:when>
				<xsl:when test="$p1 = 'xsd:choice'"><xsl:value-of select="$p2"/></xsl:when>
				<xsl:when test="$p1 = 'xsd:schema'"><xsl:value-of select="$p1"/></xsl:when>
				<xsl:when test="contains($p1,'xsd:')"><xsl:value-of select="$p2"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$p1"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="itemName">
			<xsl:call-template name="getName">
				<xsl:with-param name="item" select="."/>
				<xsl:with-param name="parent" select="$ancestor"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:if test="not(parent::node()/name()=$ancestor)"><xsl:message>A[<xsl:value-of select="$itemName"/>][<xsl:value-of select="parent::node()/name()"/>][<xsl:value-of select="$ancestor"/>]</xsl:message></xsl:if -->
		<xsl:variable name="baseType">
		       <xsl:choose>
			<xsl:when test="./@xsd:resolvedType"><type><xsl:value-of select="./@xsd:resolvedType"/></type></xsl:when>
			<xsl:when test="./@xsd:base1">
				<xsl:for-each select="./@xsd:*">
					<xsl:if test="starts-with(name(),'xsd:base')">
						<type><xsl:value-of select="."/></type>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
		    <xsl:when test="./@xsd:base"><type><xsl:value-of select="./@xsd:base"/></type></xsl:when>
			<xsl:when test="./@xsd:type|./@xs:type"><type><xsl:value-of select="./@xsd:type|./@xs:type"/></type></xsl:when>
			<xsl:otherwise/>
		 </xsl:choose>
		</xsl:variable>
		<!-- xsl:message>item <xsl:value-of select="$itemName"/>: [<xsl:copy-of select="$baseType"/>][<xsl:copy-of select="./@xsd:*"/>]</xsl:message -->
		<xsl:if test="count(./@xsd:*) &gt; 0">
			<xsl:call-template name="occurs">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minOccurs" select="./@xsd:minOccurs[1]"/>
				<xsl:with-param name="maxOccurs" select="./@xsd:maxOccurs[1]"/>
			</xsl:call-template>
			<xsl:variable name="allFactors" select="@xsd:*[contains(name(),'Digit')] | @xsd:*[contains(name(),'pattern')]"/>
			
			<xsl:variable name="allFactorAttributes"><xsl:for-each select="@xsd:*"><xsl:if test="contains(name(),'Digit') or(contains(name(),'pattern'))"><xsl:element name="{name()}"><xsl:attribute name="value"><xsl:value-of select="."/></xsl:attribute></xsl:element></xsl:if></xsl:for-each></xsl:variable>			
			<!-- xsl:if test="count($allFactorAttributes) &gt; 0"><xsl:message><xsl:value-of select="$itemName"/>[<xsl:copy-of select="$allFactorAttributes"/>]</xsl:message></xsl:if -->
			
			<xsl:variable name="factorsList" select="@xsd:*[contains(name(),'Digit')] | @xsd:*[contains(name(),'pattern')]"/>
			<xsl:variable name="typesCount" select="count($baseType/type)"/>
			<xsl:for-each select="$baseType/type">
				<xsl:choose>
					<xsl:when test="$typesCount &gt; 1"><!-- called more than once -->
							<xsl:call-template name="resolveType">
								<xsl:with-param name="itemName" select="$itemName"/>
								<xsl:with-param name="type" select="."/>
								<xsl:with-param name="factors" select="$factorsList"/>
								<xsl:with-param name="attributes" select="$allFactorAttributes"/>
								<xsl:with-param name="union" select="'true'"/>
								<xsl:with-param name="chkSimpleType"/>
								<xsl:with-param name="sequence" select="position()"/>
							</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="resolveType">
							<xsl:with-param name="itemName" select="$itemName"/>
							<xsl:with-param name="type" select="."/>
							<xsl:with-param name="factors" select="$factorsList"/>
							<xsl:with-param name="attributes" select="$allFactorAttributes"/>
							<xsl:with-param name="union" select="'false'"/>
							<xsl:with-param name="chkSimpleType"/>
							<xsl:with-param name="sequence" select="position()"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
			
			<xsl:call-template name="nillable">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="nillable" select="./@xsd:nillable"/>
			</xsl:call-template>
			
			<xsl:call-template name="setLength">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minLength" select="./@xsd:minLength[1]"/>
				<xsl:with-param name="maxLength" select="./@xsd:maxLength[1] | ./@xsd:length[1]"/>
			</xsl:call-template>
			
			<xsl:call-template name="setRange">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minValue" select="./@xsd:minInclusive[1]"/>
				<xsl:with-param name="maxValue" select="./@xsd:maxInclusive[1]"/>
			</xsl:call-template>
			
			<xsl:call-template name="setDefault">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="defaultValue" select="./@xsd:default[1]"/>
			</xsl:call-template>
			
			<xsl:if test="not($baseType = 'xsd:date' or($baseType = 'xs:date'))">
			   <xsl:for-each select="./@xsd:pattern">
				   <xsl:call-template name="setPattern">
					<xsl:with-param name="itemName" select="$itemName"/>
					<xsl:with-param name="maskValue" select="."/>
				   </xsl:call-template>
			   </xsl:for-each>
			</xsl:if>
			
		</xsl:if>
		<xsl:if test="name()='xsd:any'">
			<xsl:variable name="parentPath" select="parent::node()/name()"/>
			<xsl:variable name="branchPath">
				<xsl:choose>
					<xsl:when test="./../../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../../self::node()"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node() and(not(./../self::node()/name() = $parentPath))"><xsl:value-of select="./../self::node()/name()"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="xpath">
				<xsl:choose>
					<xsl:when test="$parentPath = 'xsd:choice'"><xsl:value-of select="concat('//',w2e:nodeParentMatch($branchPath,$p1,$p2,$p3,$p4,$p5),'/',$branchPath)"/></xsl:when>
					<xsl:when test="$branchPath = 'xsd:choice'"><xsl:value-of select="concat('//',w2e:nodeParentMatch($parentPath,$p1,$p2,$p3,$p4,$p5),'/',$parentPath)"/></xsl:when>
					<xsl:when test="$branchPath=''"><xsl:value-of select="concat('//',$parentPath)"/></xsl:when>
					<xsl:when test="$parentPath=''"><xsl:value-of select="concat('//',$branchPath)"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat('//',$branchPath,'/',$parentPath)"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">datatype(<xsl:value-of select="$xpath"/>,any)</xsl:attribute>
			</xsl:element>
		</xsl:if>
		<xsl:apply-templates select="@*" mode="rules"/>
		<xsl:apply-templates select="*" mode="rules"/>
	</xsl:template>

	<xsl:template match="xsd:choice" mode="rules">
		<xsl:param name="thisparent"/>
		<xsl:variable name="parent" select="if (string-length($thisparent) = 0) then parent::node()/name() else $thisparent"/>
		<xsl:variable name="p1" select="parent::node()/name()"/>
		<xsl:variable name="p2" select="../parent::node()/name()"/>
		<xsl:variable name="p3" select="../../parent::node()/name()"/>
		<xsl:variable name="p4" select="../../../parent::node()/name()"/>
		<xsl:variable name="p5" select="../../../../parent::node()/name()"/>
		<xsl:variable name="ancestor">
		<xsl:choose>
			<xsl:when test="$p1='xsd:choice' and($p2='xsd:choice')"><xsl:value-of select="$p3"/></xsl:when>
			<xsl:when test="$p1 = 'xsd:choice'"><xsl:value-of select="if($p2=$thisparent) then if($p3='xsd:choice') then $p4 else $p3 else $p2"/></xsl:when>
			<xsl:when test="$p2 = 'xsd:choice'"><xsl:value-of select="if($p3='xsd:choice' or($p3=$thisparent)) then $p4 else $p3"/></xsl:when>
			<xsl:when test="$p2 = $thisparent"><xsl:value-of select="if($p3='xsd:choice' or($p3=$thisparent)) then $p4 else $p3"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$p2"/></xsl:otherwise>
		</xsl:choose>
		</xsl:variable>
		<xsl:variable name="parentPath" select="parent::node()/name()"/>
		<xsl:variable name="branchPath">
			<xsl:choose>
				<xsl:when test="./../../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../../self::node()/name()"/></xsl:when>
				<xsl:when test="./../../self::node()"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
				<xsl:when test="./../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
				<xsl:when test="./../self::node() and(not(./../self::node()/name() = $parentPath))"><xsl:value-of select="./../self::node()/name()"/></xsl:when>
			</xsl:choose>
		</xsl:variable>
		
		<!-- xsl:message>choice[<xsl:value-of select="$parent"/>][<xsl:value-of select="$ancestor"/>] - [<xsl:value-of select="concat($p1,'-',$p2,'-',$p3,'-',$p4)"/> ]</xsl:message -->
		<xsl:variable name="quote">'</xsl:variable>
		<xsl:choose>
			<xsl:when test="xsd:choice">
				<xsl:apply-templates select="*" mode="rules">
					<xsl:with-param name="thisparent" select="if($ancestor='xsd:choice') then $parent else concat($ancestor,'/',$parent)"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="maxOccursValue" select="xsd:maxOccurs/text()"/>
				<xsl:variable name="choiceXPath" ><xsl:variable name="temp"><xsl:for-each select="*[not(name()='xsd:any') and(not(name()='xsd:maxOccurs'))]"><xsl:value-of select="if(contains(name(),':')) then 'contains(name(),' else '(name()='"/><xsl:value-of select="$quote" disable-output-escaping="yes"/><xsl:value-of select="name()"/><xsl:value-of select="$quote" disable-output-escaping="yes"/>) or </xsl:for-each></xsl:variable>//<xsl:value-of select="$parent"/>/*<xsl:value-of select="if (string-length($temp) &gt; 0) then concat('[',substring($temp,1,string-length($temp)-4),']') else ''" disable-output-escaping="yes"/></xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setChoice(<xsl:value-of select="replace($choiceXPath,'///','//')" disable-output-escaping="yes" />)</xsl:attribute>
				</xsl:element>
				<xsl:for-each select="*[not(name()='xsd:any') and(not(name()='xsd:maxOccurs'))]">
					<!-- xsl:message>makeOptional(//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/>)</xsl:message -->
					<xsl:element name="as:constraint">
						<xsl:attribute name="action">makeOptional(//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/>)</xsl:attribute>
					</xsl:element>
					<!-- xsl:message> - maxOccursValue - <xsl:value-of select="$maxOccursValue"/></xsl:message -->
					<xsl:if test="$maxOccursValue = 'unbounded' or($maxOccursValue &gt; '1')">
						<!-- xsl:message>makeRepeatable(//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/>)</xsl:message -->
						<xsl:element name="as:constraint">
							<xsl:attribute name="action">makeRepeatable(//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/>)</xsl:attribute>
						</xsl:element>
						<xsl:if test="not($maxOccursValue = 'unbounded')">
							<!-- xsl:message> REPEAT <xsl:value-of select="$maxOccursValue"/></xsl:message -->
							<xsl:element name="as:constraint">
								<xsl:attribute name="action">setLimit(//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/>,<xsl:value-of select="$maxOccursValue"/>)</xsl:attribute>
							</xsl:element>
						</xsl:if>
					</xsl:if>
				</xsl:for-each>			
				<xsl:apply-templates select="*" mode="rules"/>
			</xsl:otherwise>
		</xsl:choose>		
	</xsl:template>

	<xsl:template match="@*" mode="rules">
		<xsl:variable name="parentPath" select="parent::node()/name()"/>
		<xsl:variable name="itemName">
			<xsl:call-template name="getAttrName">
				<xsl:with-param name="item" select="."/>
				<xsl:with-param name="parent" select="parent::node()/name()"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:message>attribute name: <xsl:value-of select="$itemName"/></xsl:message -->
		<xsl:variable name="attr-stub">
			<xsl:value-of select="concat('attr-',./local-name(),'-')"/>
		</xsl:variable>
		<!-- xsl:message>
			<xsl:value-of select="concat($attr-stub,':-',count(parent::node()/@xsd:*[starts-with(name(),$attr-stub)]))"/>
		</xsl:message -->
		<!-- xsl:message>
			<xsl:value-of select="concat($attr-stub,'::',count(parent::node()/@*[contains(name(),$attr-stub)]))"/>
		</xsl:message -->

		<xsl:if test="count(parent::node()/@xsd:*[starts-with(local-name(),$attr-stub)]) &gt; 0">
			<!-- xsl:message>- use: <xsl:value-of select="local-name()"/> - <xsl:value-of select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'use'))]"/></xsl:message -->
			<xsl:variable name="minOccurs">
				<xsl:variable name="use" select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'use'))]"/>
				<xsl:value-of select="if($use='required') then 1 else if($use='optional') then 0 else 0"/>
			</xsl:variable>
			<xsl:call-template name="occurs">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minOccurs" select="$minOccurs"/>
				<xsl:with-param name="maxOccurs" select="1"/>
			</xsl:call-template>
			
			<xsl:variable name="filterAttrs" select="substring-after($itemName,'@')"/>
			<xsl:variable name="allFactorAttributes"><xsl:for-each select="parent::node()/@xsd:*"><xsl:if test="contains(name(),'Digit') or(contains(name(),'pattern'))"><xsl:if test="contains(name(),concat('-',$filterAttrs,'-'))"><xsl:element name="{name()}"><xsl:attribute name="value"><xsl:value-of select="."/></xsl:attribute></xsl:element></xsl:if></xsl:if></xsl:for-each></xsl:variable>
			<!-- xsl:if test="count($allFactorAttributes) &gt; 0 and($filterAttrs='NumberValue')"><xsl:message>attr: <xsl:value-of select="$filterAttrs"/>[<xsl:copy-of select="parent::node()/@xsd:*"/>]</xsl:message></xsl:if -->
			<xsl:variable name="attr-type-value" select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'type'))]"/>
			<xsl:variable name="chkSimpleType">
				<xsl:choose>
					<xsl:when test="count($entireXSDcollection//xsd:simpleType[@name=$attr-type-value]/xsd:restriction) &gt; 0">
						<xsl:variable name="simpleTypeMatch" select="$entireXSDcollection//xsd:simpleType[@name=$attr-type-value]"/>
						<!-- find namespace details -->
						<xsl:variable name="defaultNSlist"><xsl:for-each select="$simpleTypeMatch"><xsl:for-each select="namespace::*"><xsl:if test="string-length(name())=0"><xsl:value-of select="."/>|</xsl:if></xsl:for-each></xsl:for-each></xsl:variable>
						<xsl:variable name="completeNSlist"><nss><xsl:for-each select="$simpleTypeMatch"><xsl:for-each select="namespace::*"><xsl:if test="string-length(name())&gt;0"><xsl:element name="ns"><xsl:attribute name="prefix" select="name()"/><xsl:attribute name="value" select="."/></xsl:element></xsl:if></xsl:for-each></xsl:for-each></nss></xsl:variable>
						<xsl:variable name="parentNSmatch">
							<xsl:for-each select="distinct-values(tokenize($defaultNSlist,'\|'))">
								<xsl:variable name="match" select="."/>
								<xsl:if test="$completeNSlist//ns[@value=$match]/@prefix"><xsl:value-of select="$completeNSlist//ns[@value=$match]/@prefix[1]"/>|</xsl:if>
							</xsl:for-each>
						</xsl:variable>
						<!-- xsl:message>type: [<xsl:value-of select="$attr-type-value"/>]-[<xsl:value-of select="count($simpleTypeMatch)"/>]-[<xsl:value-of select="$parentNSmatch"/>]</xsl:message -->
						<!-- match to corresponding namespace or default -->
						<xsl:choose>
							<xsl:when test="count($simpleTypeMatch) &lt; 2"><xsl:value-of select="$simpleTypeMatch[1]/xsd:restriction/@base"/></xsl:when>
							<xsl:when test="contains($attr-type-value,':')">
								<xsl:if test="contains($parentNSmatch,concat(substring-before($attr-type-value,':'),'|'))">
									<xsl:variable name="typeNSlist" select="tokenize($parentNSmatch,'\|')"/>
									<xsl:variable name="attrNS" select="substring-before($attr-type-value,':')"/>
									<xsl:variable name="matchNSpos">
										<xsl:for-each select="$typeNSlist"><xsl:if test=".=$attrNS"><xsl:element name="match"><xsl:value-of select="position()"/></xsl:element></xsl:if></xsl:for-each>
									</xsl:variable>
									<xsl:value-of select="$simpleTypeMatch[$matchNSpos/match[1]]/xsd:restriction/@base"/>
								</xsl:if>
							</xsl:when>
							<xsl:otherwise><xsl:value-of select="$simpleTypeMatch[1]/xsd:restriction/@base"/></xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:when test="count($entireXSDcollection//xsd:simpleType[@name=$attr-type-value]/xsd:union) &gt; 0">
						<xsl:variable name="simpleUnionMatch" select="$entireXSDcollection//xsd:simpleType[@name=$attr-type-value]/xsd:union"/>
						<xsl:variable name="prefix2">
							<xsl:call-template name="prefix-for-namespace">
								<xsl:with-param name="namespace-uri" select="$simpleUnionMatch[1]//namespace::*"/>
								<xsl:with-param name="context" select="$simpleUnionMatch[1]"/>
							</xsl:call-template>
						</xsl:variable>
						<!-- xsl:message>union: [<xsl:copy-of select="$simpleUnionMatch"/>] ns: [<xsl:value-of select="$simpleUnionMatch//namespace::*"/>]prefix: [<xsl:value-of select="$prefix2"/>]</xsl:message -->
						<xsl:value-of select="$simpleUnionMatch[1]/@memberTypes"/>
					</xsl:when>
					<xsl:otherwise><xsl:value-of select="$attr-type-value"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<!-- xsl:message>simpleType [<xsl:value-of select="$chkSimpleType"/>]</xsl:message -->
			<xsl:call-template name="resolveType">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="type" select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'type'))]"/>
				<xsl:with-param name="factors" select="parent::node()/@xsd:*[contains(name(),'Digit')] | parent::node()/@xsd:*[contains(name(),'pattern')]"></xsl:with-param>
				<xsl:with-param name="attributes" select="$allFactorAttributes"/>
				<xsl:with-param name="union" select="if(contains($chkSimpleType,' ')) then 'true' else 'false'"/>
				<xsl:with-param name="chkSimpleType" select="$chkSimpleType"/>
				<xsl:with-param name="sequence" select="1"/>
			</xsl:call-template>
			<xsl:if test="contains($chkSimpleType,' ')"><!-- Called more than once for union -->
				<xsl:call-template name="resolveType">
					<xsl:with-param name="itemName" select="$itemName"/>
					<xsl:with-param name="type" select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'type'))]"/>
					<xsl:with-param name="factors" select="parent::node()/@xsd:*[contains(name(),'Digit')] | parent::node()/@xsd:*[contains(name(),'pattern')]"></xsl:with-param>
					<xsl:with-param name="attributes" select="$allFactorAttributes"/>
					<xsl:with-param name="union" select="'true'"/>
					<xsl:with-param name="chkSimpleType" select="$chkSimpleType"/>
					<xsl:with-param name="sequence" select="2"/>
				</xsl:call-template>
			</xsl:if>

			<xsl:call-template name="nillable">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="nillable" select="parent::node()/@xsd:*[contains(local-name(),concat($attr-stub,'nillable'))]"/>
			</xsl:call-template>
			
			<xsl:call-template name="setRange">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minValue" select="parent::node()/@xsd:*[contains(name(),concat($attr-stub,'minInclusive'))]"/>
				<xsl:with-param name="maxValue" select="parent::node()/@xsd:*[contains(name(),concat($attr-stub,'maxInclusive'))]"/>
			</xsl:call-template>
			
			<xsl:call-template name="setPattern">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="maskValue" select="parent::node()/@xsd:*[contains(name(),concat($attr-stub,'pattern'))]"/>
			</xsl:call-template>
			
			<xsl:call-template name="setLength">
				<xsl:with-param name="itemName" select="$itemName"/>
				<xsl:with-param name="minLength" select="parent::node()/@xsd:*[contains(name(),concat($attr-stub,'minLength'))]"/>
				<xsl:with-param name="maxLength" select="parent::node()/@xsd:*[contains(name(),concat($attr-stub,'maxLength'))]"/>
			</xsl:call-template>
		</xsl:if>
		
	</xsl:template>

	<xsl:template match="@xsd:*" mode="rules">
		<xsl:variable name="p1" select="parent::node()/name()"/>
		<xsl:variable name="p2" select="../parent::node()/name()"/>
		<xsl:variable name="p3" select="../../parent::node()/name()"/>
		<xsl:variable name="p4" select="../../../parent::node()/name()"/>
		<xsl:variable name="p5" select="../../../../parent::node()/name()"/>
		<xsl:if test="contains(name(),'-enumValues')">
			<xsl:variable name="parentPath" select="parent::node()/name()"/>
			<xsl:variable name="branchPath">
				<xsl:choose>
					<xsl:when test="./../../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../../self::node()"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node() and(not(./../self::node()/name() = $parentPath))"><xsl:value-of select="./../self::node()/name()"/></xsl:when>
				</xsl:choose>
			</xsl:variable>	
			<xsl:variable name="enumName" select="substring-before(substring-after(local-name(),'attr-'),'-enumValues')"/>
			<!-- xsl:message> | <xsl:value-of select="$branchPath"/> | <xsl:value-of select="$parentPath"/> | <xsl:value-of select="$p1"/>,<xsl:value-of select="$p2"/>,<xsl:value-of select="$p3"/>,<xsl:value-of select="$p4"/></xsl:message -->
			<xsl:variable name="xpath">
				<xsl:choose>
					<xsl:when test="$parentPath = 'xsd:choice'"><xsl:value-of select="concat('//',w2e:nodeParentMatch($branchPath,$p1,$p2,$p3,$p4,$p5),'/',$branchPath)"/></xsl:when>
					<xsl:when test="$branchPath = 'xsd:choice'"><xsl:value-of select="concat('//',w2e:nodeParentMatch($parentPath,$p1,$p2,$p3,$p4,$p5),'/',$parentPath)"/></xsl:when>
					<xsl:when test="$branchPath=''"><xsl:value-of select="concat('//',$parentPath)"/></xsl:when>
					<xsl:when test="$parentPath=''"><xsl:value-of select="concat('//',$branchPath)"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat('//',$branchPath,'/',$parentPath)"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="itemPath">
				<xsl:choose>
					<xsl:when test="contains(name(),'-enumValues-E')"><xsl:value-of select="$xpath"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat($xpath,'/@',$enumName)"/></xsl:otherwise>
				</xsl:choose>	</xsl:variable> 
					<!-- xsl:message><xsl:value-of select="$enumName"/> - enum-values: [<xsl:value-of select="."/>]</xsl:message -->
					<!-- xsl:variable name="matched" select="parent::node()/@xsd:*[contains(local-name(),concat($enumName,'-enumValues'))]"></xsl:variable -->
					<xsl:call-template name="enumValues">
						<xsl:with-param name="itemName" select="$itemPath"/>
						<xsl:with-param name="values" select="."/>
						<xsl:with-param name="enumName" select="concat('-',$enumName,'-')"/>
						<xsl:with-param name="annoChk" select="../attribute::*"/>
					</xsl:call-template>
		</xsl:if>
		<xsl:if test="local-name()='anyAttribute'">
			<xsl:variable name="parentPath" select="parent::node()/name()"/>
			<xsl:variable name="branchPath">
				<xsl:choose>
					<xsl:when test="./../../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../../self::node()"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node()/name() = 'xsd:choice'"><xsl:value-of select="./../../self::node()/name()"/></xsl:when>
					<xsl:when test="./../self::node() and(not(./../self::node()/name() = $parentPath))"><xsl:value-of select="./../self::node()/name()"/></xsl:when>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="itemPath"><xsl:value-of select="if (string-length($branchPath) > 1 and(not($branchPath='xsd:choice'))) then concat('//',$branchPath,'/',$parentPath) else concat('//',$parentPath)"/></xsl:variable>
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">datatype(<xsl:value-of select="$itemPath"/>,anyAttribute)</xsl:attribute>
			</xsl:element>
		</xsl:if>
	</xsl:template>

	<xsl:template name="putChoice">
		<xsl:param name="nodename"/>
		<xsl:param name="childname"/>
		<xsl:param name="excludename"></xsl:param>
		<xsl:element name="as:constraint">
			<xsl:variable name="choiceList">setChoice(/<xsl:if test="string-length($nodename) &gt; 0"><xsl:value-of select="$nodename"/></xsl:if>/<xsl:if test="string-length($childname) &gt; 0"><xsl:value-of select="$childname"/>/</xsl:if>*[not(contains(name(),'<xsl:value-of select="$excludename"/>'))])</xsl:variable>
			<xsl:attribute name="action"><xsl:value-of select="replace($choiceList,'///','//')"/></xsl:attribute>
		</xsl:element>
	</xsl:template>

	<xsl:template name="getName">
		<xsl:param name="item"/>
		<xsl:param name="parent"/>
		<xsl:choose>
			<xsl:when test="contains($parent,'xsd:')">//<xsl:if test="not(contains($item/name(),':'))"></xsl:if><xsl:value-of select="$item/name()"/></xsl:when><!-- removed asm1: namespace prefix -->
			<xsl:otherwise>//<xsl:if test="string-length($parent) &gt; 0"><xsl:value-of select="$parent"/>/</xsl:if><xsl:if test="not(contains($item/name(),':'))"></xsl:if><xsl:value-of select="$item/name()"/></xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template name="getAttrName">
		<xsl:param name="item"/>
		<xsl:param name="parent"/>
		<xsl:if test="string-length($parent) &gt; 0">//<xsl:value-of select="$parent"/></xsl:if>/@<xsl:value-of select="$item/name()"/>
	</xsl:template>

	<xsl:template name="resolveType">
		<xsl:param name="itemName"/>
		<xsl:param name="type"/>
		<xsl:param name="factors"/>
		<xsl:param name="attributes"/>
		<xsl:param name="union"/>
		<xsl:param name="chkSimpleType"/>
		<xsl:param name="sequence"/>
		<xsl:variable name="w3c-types" select="'|anyURI|base64Binary|byte|double|duration|ENTITIES|ENTITY|float|hexBinary|ID|IDREF|IDREFS|language|long|Name|NCName|negativeInteger|NMTOKEN|NMTOKENS|nonPositiveInteger|normalizedString|NOTATION|Qname|short|token|unsignedByte|unsignedInt|unsignedLong|unsignedShort|QName'"/>
		<xsl:variable name="typeDef">
			<xsl:choose>
				<xsl:when test="$union='true' and(contains($chkSimpleType,'float'))">xsd:float</xsl:when><!-- if union of numeric types - pick float mask -->
				<xsl:when test="$union='true' and(contains($chkSimpleType,'decimal'))">xsd:decimal</xsl:when><!-- if union of numeric types - pick decimal mask -->
				<xsl:when test="string-length($chkSimpleType) = 0"><xsl:value-of select="if(contains($type,':')) then concat('xsd:',substring-after($type,':')) else $type"/></xsl:when>
				<xsl:when test="$type=$chkSimpleType"><xsl:value-of select="if(contains($type,':')) then concat('xsd:',substring-after($type,':')) else $type"/></xsl:when>
				<xsl:otherwise>
					<xsl:choose><!-- if union pick first type or second type - otherwise type -->
						<xsl:when test="contains($chkSimpleType,' ') and $sequence &lt; 2"><xsl:value-of select="substring-before($chkSimpleType,' ')"/></xsl:when>
						<xsl:when test="contains($chkSimpleType,' ') and $sequence &gt; 1"><xsl:value-of select="substring-after($chkSimpleType,' ')"/></xsl:when>
						<xsl:otherwise><xsl:value-of select="$chkSimpleType"/></xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="$union='true'"><xsl:value-of select="w2e:message(concat(' - resolveType: ',$itemName,' + [',$type,'] - [',$typeDef,'] - union: ',$union, ' seqn: ',$sequence),4)"/></xsl:if>
		<xsl:variable name="w3c-chk" select="concat('|',if(contains($type,':')) then substring-after($type,':') else $type,'|')"/>
		<!-- xsl:value-of select="w2e:message(concat('resolveType: ',$itemName,' + [',$type,'] - [',$typeDef,'] - union: ',$union),1)"/ -->
		
		<xsl:choose>
			<xsl:when test="$typeDef = 'xsd:dateTime' or($typeDef = 'xs:dateTime')">
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &lt;26</xsl:attribute -->
					<xsl:choose>
						<xsl:when test="$union='true'">
							<xsl:attribute name="condition">not(matches(.<xsl:value-of select="if(contains($itemName,'@')) then concat('/@',substring-after($itemName,'@')) else ''"/>,'(\d\.\d)')) and(string-length(.) &gt; 19)</xsl:attribute>
						</xsl:when>
						<xsl:otherwise><xsl:attribute name="condition">not(matches(.<xsl:value-of select="if(contains($itemName,'@')) then concat('/@',substring-after($itemName,'@')) else ''"/>,'(\d\.\d)'))</xsl:attribute></xsl:otherwise>
					</xsl:choose>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY-MM-DD'T'HH:MI:SSZ)</xsl:attribute>
				</xsl:element>
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &gt; 25</xsl:attribute -->
					<xsl:attribute name="condition">matches(.<xsl:value-of select="if(contains($itemName,'@')) then concat('/@',substring-after($itemName,'@')) else ''"/>,'(\d\.\d)')</xsl:attribute>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY-MM-DD'T'HH:MI:SS.SZ)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:date' or($typeDef = 'xs:date')">
				<xsl:variable name="dateAttr">
					<xsl:choose>
						<xsl:when test="string-length($attributes) &gt; 2"><xsl:value-of select="$attributes"/></xsl:when>
						<xsl:when test="string-length($factors) &gt; 2"><xsl:value-of select="$factors"/></xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:variable>
				<!-- xsl:message>weird date: [<xsl:value-of select="$factors"/>][<xsl:value-of select="$attributes"/>][<xsl:value-of select="$dateAttr"/>]</xsl:message -->
				<xsl:choose>
				<xsl:when test="string-length($dateAttr) &gt; 0">
					<xsl:element name="as:constraint">
						<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,<xsl:value-of select="translate($dateAttr,'C','Y')"/>)</xsl:attribute>
					</xsl:element>
				</xsl:when>
				<xsl:otherwise>
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &lt;11</xsl:attribute -->
					<xsl:attribute name="condition">string-length(.) &lt;11</xsl:attribute>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY-MM-DD)</xsl:attribute>
				</xsl:element>
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &gt;10</xsl:attribute -->
					<xsl:choose>
						<xsl:when test="$union='true'"><xsl:attribute name="condition">string-length(.) &gt;10 and(string-length(.) &lt;20)</xsl:attribute></xsl:when>
						<xsl:otherwise><xsl:attribute name="condition">string-length(.) &gt;10</xsl:attribute></xsl:otherwise>
					</xsl:choose>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY-MM-DDZ)</xsl:attribute>
				</xsl:element>
				</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:time' or($typeDef = 'xs:time')">
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &lt;13</xsl:attribute -->
					<xsl:attribute name="condition">string-length(.) &lt;13</xsl:attribute>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,HH:MI:SS.SSS)</xsl:attribute>
				</xsl:element>
				<xsl:element name="as:constraint">
					<!-- xsl:attribute name="condition">string-length(<xsl:value-of select="$itemName"/>) &gt;12</xsl:attribute -->
					<xsl:attribute name="condition">string-length(.) &gt;12</xsl:attribute>
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,HH:MI:SS.SSSZ)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:gYear' or($typeDef = 'xs:gYear')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:gYearMonth' or($typeDef = 'xs:gYearMonth')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,YYYY-MM)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$type = 'xsd:gMonth' or($type = 'xs:gMonth')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,'--'MM)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:gMonthDay' or($typeDef = 'xs:gMonthDay')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,'--'MM-DD)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:gDay' or($typeDef = 'xs:gDay')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setDateMask(<xsl:value-of select="$itemName"/>,'---'DD)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef= 'xsd:boolean' or($typeDef = 'xs:boolean')">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">restrictValues(<xsl:value-of select="$itemName"/>,'true'|'false')</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="contains($typeDef,'xsd:duration') or(contains($typeDef,'xs:duration'))">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">restrictValues(<xsl:value-of select="$itemName"/>,'P1'|'Y2'|'M3'|'DT1'|'H1'|'0M'|'0S')</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:decimal' or($typeDef = 'xs:decimal')">
				<xsl:variable name="mask">
					<xsl:call-template name="computeMask">
						<xsl:with-param name="type">decimal</xsl:with-param>
						<xsl:with-param name="factors" select="$factors"/>
						<xsl:with-param name="attributes" select="$attributes"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setNumberMask(<xsl:value-of select="concat($itemName,',',$mask)"/>)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:float' or($typeDef = 'xs:float')">
				<xsl:variable name="mask">
					<xsl:call-template name="computeMask">
						<xsl:with-param name="type">float</xsl:with-param>
						<xsl:with-param name="factors" select="$factors"/>
						<xsl:with-param name="attributes" select="$attributes"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setNumberMask(<xsl:value-of select="concat($itemName,',',$mask)"/>)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:integer' or($typeDef = 'xs:integer') or($typeDef = 'xsd:int') or($typeDef = 'xs:int') or($typeDef = 'integer') or($typeDef = 'int')">
				<xsl:variable name="mask">
					<xsl:call-template name="computeMask">
						<xsl:with-param name="type">integer</xsl:with-param>
						<xsl:with-param name="factors" select="$factors"/>
						<xsl:with-param name="attributes" select="$attributes"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setNumberMask(<xsl:value-of select="concat($itemName,',',$mask)"/>)</xsl:attribute>
				</xsl:element>			
			</xsl:when>
			<xsl:when test="$typeDef = 'xsd:nonNegativeInteger' or($typeDef = 'xs:nonNegativeInteger')  or($typeDef = 'nonNegativeInteger') or($typeDef = 'xs:positiveInteger') or($typeDef = 'xsd:positiveInteger') or($typeDef = 'positiveInteger')">
				<xsl:variable name="mask">
					<xsl:call-template name="computeMask">
						<xsl:with-param name="type">pos-integer</xsl:with-param>
						<xsl:with-param name="factors" select="$factors"/>
						<xsl:with-param name="attributes" select="$attributes"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setNumberMask(<xsl:value-of select="concat($itemName,',',$mask)"/>)</xsl:attribute>
				</xsl:element>
				<xsl:call-template name="setRange">
					<xsl:with-param name="itemName" select="$itemName"/>
					<xsl:with-param name="minValue" select="$factors/@xsd:minInclusive[1]"/>
					<xsl:with-param name="maxValue" select="$factors/@xsd:maxInclusive[1]"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="$factors/@xsd:maxInclusive[1]">
				<xsl:variable name="maxValue" select="xsd:integer($factors/@xsd:maxInclusive[1])"/>
				<xsl:if test="$maxValue &gt; 0">
					<xsl:variable name="digits" select="string-length($factors/@xsd:maxInclusive[1])"/>
					<xsl:element name="as:constraint">
						<xsl:attribute name="action">setNumberMask(<xsl:value-of select="concat($itemName,',','#',$digits)"/>)</xsl:attribute>
					</xsl:element>
				</xsl:if>
			</xsl:when>
			<xsl:when test="contains($w3c-types,$w3c-chk)">
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">datatype(<xsl:value-of select="$itemName"/>,<xsl:value-of select="if(contains($type,':')) then substring-after($type,':') else $type"/>)</xsl:attribute>
				</xsl:element>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="occurs">
		<xsl:param name="itemName"/>
		<xsl:param name="minOccurs"/>
		<xsl:param name="maxOccurs"/>
		<xsl:if test="count($maxOccurs) &gt; 0 and(not($maxOccurs ='0')) and(not($maxOccurs ='1'))">
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">makeRepeatable(<xsl:value-of select="$itemName"/>)</xsl:attribute>
			</xsl:element>
		</xsl:if>
		<xsl:if test="count($maxOccurs) &gt; 0 and($maxOccurs &gt; '1') and(not($maxOccurs='unbounded'))">
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">setLimit(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$maxOccurs"/>)</xsl:attribute>
			</xsl:element>
		</xsl:if>
		<xsl:if test="$minOccurs = '0'">
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">makeOptional(<xsl:value-of select="$itemName"/>)</xsl:attribute>
			</xsl:element>
		</xsl:if>
	</xsl:template>

	<xsl:template name="setLength">
		<xsl:param name="itemName"/>
		<xsl:param name="minLength"/>
		<xsl:param name="maxLength"/>
		<xsl:if test="count($minLength) &gt;0 or(count($maxLength) &gt;0)">
		<xsl:variable name="minLen">
			<xsl:choose>
				<xsl:when test="count($minLength)=0">0</xsl:when>
				<xsl:when test="string-length($minLength)=0">0</xsl:when>
				<xsl:otherwise><xsl:value-of select="$minLength"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="maxLen">
			<xsl:choose>
				<xsl:when test="count($maxLength)=0">32000</xsl:when>
				<xsl:when test="string-length($maxLength)=0">32000</xsl:when>
				<xsl:otherwise><xsl:value-of select="$maxLength"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:element name="as:constraint">
			<xsl:attribute name="action">setLength(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$minLen"/>-<xsl:value-of select="$maxLen"/>)</xsl:attribute>
		</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="setRange">
		<xsl:param name="itemName"/>
		<xsl:param name="minValue"/>
		<xsl:param name="maxValue"/>
		<xsl:choose>
			<xsl:when test="contains($minValue,'-')"/><!-- skip if date minimum value -->
			<xsl:when test="count($minValue) &gt;0 or(count($maxValue) &gt;0)">
				<xsl:variable name="minVal">
					<xsl:choose>
						<xsl:when test="count($minValue)=0">0</xsl:when>
						<xsl:when test="string-length($minValue)=0">0</xsl:when>
						<xsl:otherwise><xsl:value-of select="$minValue"/></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:variable name="maxVal">
					<xsl:choose>
						<xsl:when test="count($maxValue)=0">32000</xsl:when>
						<xsl:when test="string-length($maxValue)=0">32000</xsl:when>
						<xsl:otherwise><xsl:value-of select="$maxValue"/></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:element name="as:constraint">
					<xsl:attribute name="action">setNumberRange(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$minVal"/>-<xsl:value-of select="$maxVal"/>)</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template name="setDefault">
		<xsl:param name="itemName"/>
		<xsl:param name="defaultValue"/>

		<xsl:if test="count($defaultValue) &gt;0">
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">setDefault(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$defaultValue"/>)</xsl:attribute>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="setPattern">
		<xsl:param name="itemName"/>
		<xsl:param name="maskValue"/>
		<xsl:if test="count($maskValue) &gt;0">
			<!-- morph regexp into CAM mask -->
			<xsl:for-each select="tokenize($maskValue,'\|\|\|')">
			   <xsl:variable name="thisMask" select="normalize-space(.)"/>
			   <xsl:if test="string-length($thisMask) &gt; 0">
			     <xsl:variable name="mask"><xsl:value-of select="w2e:regexp-2-mask($thisMask)"/></xsl:variable>
			     <xsl:element name="as:constraint">
				<xsl:attribute name="action">setStringMask(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$mask"/>)</xsl:attribute>
			     </xsl:element>
			   </xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>

	<xsl:template name="nillable">
		<xsl:param name="itemName"/>
		<xsl:param name="nillable"/>
		<xsl:if test="$nillable = 'true'">
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">makeNillable(<xsl:value-of select="$itemName"/>,xsd)</xsl:attribute>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="enumValues">
		<xsl:param name="itemName"/>
		<xsl:param name="values"/>
		<xsl:param name="enumName"/>
		<xsl:param name="annoChk"/>
		<!-- xsl:if test="string-length($values) &gt; 1" -->
			<xsl:element name="as:constraint">
				<xsl:attribute name="action">restrictValues(<xsl:value-of select="$itemName"/>,<xsl:value-of select="$values"/>)</xsl:attribute>
				<xsl:if test="contains($CAMmode,'anno') and($annoChk[contains(name(),'enumAnno')])">	
					<xsl:element name="as:annotation">
						<xsl:element name="as:documentation"><xsl:attribute name="type">Definition</xsl:attribute>
							<xsl:for-each select="$annoChk">
								<xsl:if test="contains(name(),'enumAnno') and(contains(name(),$enumName))">
									<xsl:value-of select="."/><xsl:value-of select="' | &#xD;'"/>
								</xsl:if>
							</xsl:for-each>
						</xsl:element>
					</xsl:element>
				</xsl:if>
			</xsl:element>
		<!-- /xsl:if -->
	</xsl:template>
	
	<xsl:template name="computeMask">
		<xsl:param name="type"/>
		<xsl:param name="factors"/>
		<xsl:param name="attributes"/>
		<!-- xsl:message><xsl:value-of select="$type"/>- factors[<xsl:value-of select="$factors"/>] -attributes[<xsl:copy-of select="$attributes"/>]</xsl:message -->
		<xsl:variable name="maxDigits" select="$attributes/*[contains(name(),'totalDigits')]/@value"/>
		<xsl:variable name="fractions" select="$attributes/*[contains(name(),'fractionDigits')]/@value"></xsl:variable>
		<xsl:variable name="digitMask">
			<xsl:choose>
				<xsl:when test="$maxDigits='0' or(string-length($maxDigits)=0)">######</xsl:when>
				<xsl:when test="not(matches($maxDigits,'[0-9]+'))">######</xsl:when>
				<xsl:when test="$maxDigits='1'">#</xsl:when>
				<xsl:when test="$maxDigits='2'">##</xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('#',$maxDigits)"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="fractionMask">
			<xsl:choose>
				<xsl:when test="$fractions='0'">
					<xsl:choose>
						<xsl:when test="$type='decimal'"></xsl:when><!-- removed . -->
						<xsl:when test="$type='float'">.</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="string-length($fractions)=0">
					<xsl:choose>
						<xsl:when test="$type='decimal'">.##</xsl:when>
						<xsl:when test="$type='float'">.####</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="not(matches($fractions,'[0-9]+'))">
					<xsl:choose>
						<xsl:when test="$type='decimal'">.##</xsl:when>
						<xsl:when test="$type='float'">.####</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="$fractions='1'">.#</xsl:when>
				<xsl:when test="$fractions='2'">.##</xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('.#',$fractions)"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$type='integer'"><xsl:value-of select="$digitMask"/></xsl:when>
			<xsl:when test="$type='pos-integer'"><xsl:value-of select="$digitMask"/></xsl:when>
			<xsl:when test="$type='decimal'"><xsl:value-of select="concat($digitMask,$fractionMask)"/></xsl:when>
			<xsl:when test="$type='float'"><xsl:value-of select="concat($digitMask,$fractionMask)"/></xsl:when>
			<xsl:otherwise>*unknown*</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="regexp-replace-patterns">
		<xsl:param name="regexp-in"/>
		<xsl:variable name="firstChar" select="substring($regexp-in,1,1)"/>
		<xsl:choose>
			<xsl:when test="normalize-space($regexp-in) = ''"/><!-- empty string -->
			<xsl:when test="contains($regexp-in,'|||')">
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="substring-before($regexp-in,'|||')"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="$firstChar='('">
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="normalize-space(substring-after($regexp-in,'('))"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="$firstChar=')'">
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="normalize-space(substring-after($regexp-in,')'))"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="$firstChar='?'">
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="normalize-space(substring-after($regexp-in,'?'))"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="$firstChar = '['">
				<xsl:variable name="maskChars" select="normalize-space(substring-after(replace(replace(substring-before($regexp-in,']'),' ',''),',',''),'['))"/>
				<xsl:variable name="remainingChars" select="substring-after($regexp-in,']')"/>
				<xsl:variable name="optional">
					<xsl:choose>
						<xsl:when test="starts-with($remainingChars,'?')">Y</xsl:when>
						<xsl:when test="contains($remainingChars,'{')">
							<xsl:variable name="repeats" select="substring-before(substring-after($remainingChars,'{'),'}')"/>
							<xsl:value-of select="if(contains($repeats,',')) then 'Y' else 'N'"/>
						</xsl:when>
						<xsl:otherwise>N</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:choose>
					<xsl:when test="$maskChars='a-z'"><xsl:value-of select="if($optional = 'Y') then '_' else 'L'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="$maskChars='A-Z'"><xsl:value-of select="if($optional = 'Y') then '^' else 'U'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="$maskChars='1-9'"><xsl:value-of select="if($optional = 'Y') then '#' else '0'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="$maskChars='0-9'"><xsl:value-of select="if($optional = 'Y') then '#' else '0'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9A-Za-z')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9|A-Za-z')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9|a-zA-Z')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9a-zA-Z')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'a-zA-Z0-9')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'A-Za-z0-9')"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9A-Z')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9|A-Z')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9a-z')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'0-9|a-z')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'A-Z0-9')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'A-Z|0-9')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'a-z0-9')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="contains($maskChars,'a-z|0-9')"><xsl:value-of select="if($optional = 'Y') then 'a' else 'A'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:when test="count(tokenize($maskChars,'-')) &gt; 3"><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<!-- xsl:when test="string-length($maskChars) &lt;3">'<xsl:value-of select="substring($maskChars,2,1)"/>'<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when -->
					<xsl:when test="string-length($maskChars) = 1">'<xsl:value-of select="$maskChars"/>'<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:otherwise><xsl:value-of select="if($optional = 'Y') then '?' else 'X'"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$firstChar = '{'">
				<xsl:variable name="repeats" select="normalize-space(substring-after(substring-before($regexp-in,'}'),'{'))"/>
				<xsl:variable name="remainingChars" select="substring-after($regexp-in,'}')"/>
				<xsl:choose>
					<xsl:when test="contains($repeats,',')"><xsl:value-of select="substring-after($repeats,',')"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:when>
					<xsl:otherwise><xsl:value-of select="$repeats"/><xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="$remainingChars"/></xsl:with-param></xsl:call-template></xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="contains($regexp-in,'[')">
				<xsl:variable name="nextChars"><xsl:value-of select="concat('''',normalize-space(substring-before($regexp-in,'[')),'''')"/></xsl:variable>
				<xsl:if test="string-length($nextChars) &gt; 2"><xsl:value-of select="$nextChars"/></xsl:if>
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="concat('[',substring-after($regexp-in,'['))"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="contains($regexp-in,'{')">
				<xsl:variable name="nextChars"><xsl:value-of select="concat('''',normalize-space(substring-before($regexp-in,'{')),'''')"/></xsl:variable>
				<xsl:if test="string-length($nextChars) &gt; 2"><xsl:value-of select="$nextChars"/></xsl:if>
				<xsl:call-template name="regexp-replace-patterns"><xsl:with-param name="regexp-in"><xsl:value-of select="concat('{',substring-after($regexp-in,'{'))"/></xsl:with-param></xsl:call-template>
			</xsl:when>
			<xsl:when test="string-length(normalize-space($regexp-in)) &lt; 1"/>
			<xsl:otherwise>'<xsl:value-of select="normalize-space($regexp-in)"/>'</xsl:otherwise>
		</xsl:choose>
		<!-- xsl:value-of select="replace(replace(replace(replace(replace($regexp-in,'\[0-9|A-Z\]','X'),'\[A-Z\]','U'),'\[0-9\]','#'),'\{',''),'\}','')"/ -->
	</xsl:template>
	
	<xsl:template name="regexp-replace-pattern">
		<xsl:param name="regexp-in"/>
		<xsl:value-of select="concat('|',$regexp-in)"/>
	</xsl:template>
	
	<xsl:function name="w2e:regexp-2-mask"><!-- parse regexp into CAM mask pattern -->
	       <xsl:param name="regexp-in"/>
	<xsl:choose>
		<xsl:when test="contains($regexp-in,$GDSC-restricts)"><xsl:value-of select="$GDSC-Mask"/></xsl:when>
		<xsl:when test="contains($regexp-in,$TelephoneNumber)"><xsl:value-of select="$TelephoneCAM"/></xsl:when>
		<xsl:when test="string-length($regexp-in) &gt; 0">
	       	<xsl:variable name="regexpCAM">
	       		<xsl:call-template name="regexp-replace-patterns">
	       			<xsl:with-param name="regexp-in"><xsl:value-of select="$regexp-in"/></xsl:with-param>
	       		</xsl:call-template>
	       	</xsl:variable>
	       	<!-- xsl:message> - mask <xsl:value-of select="$regexpCAM"/> - <xsl:value-of select="$regexp-in"/></xsl:message -->
	       	<!-- adjust incoming regexp for bracket pairs -->
		<!-- xsl:variable name="regexp">
			<xsl:choose>
				<xsl:when test="starts-with($regexpCAM,'([')">
					<xsl:variable name="mask1">
						<xsl:choose>
							<xsl:when test="contains($regexpCAM,']){')"><xsl:value-of select="concat(substring-after(replace(substring-before($regexpCAM,'}'),'\]\)',']'),'('),'}')"/></xsl:when>
							<xsl:when test="contains($regexpCAM,'])')"><xsl:value-of select="concat(substring-after(substring-before($regexpCAM,'\]\)'),'('),']')"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="substring-after($regexpCAM,'(')"/></xsl:otherwise>
						</xsl:choose>
					</xsl:variable>
					<xsl:value-of select="$mask1"/>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$regexpCAM"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable -->
	       	<!-- find if there is a second mask to process next -->
		<!-- xsl:variable name="mask2">
			<xsl:choose>
				<xsl:when test="contains($regexp-in,']){')"><xsl:value-of select="substring-after(substring-after($regexp-in,']){'),'}')"/></xsl:when>
				<xsl:when test="contains($regexp-in,'])')"><xsl:value-of select="substring-after($regexp-in,'])')"/></xsl:when>
				<xsl:when test="contains($regexp-in,']{')"><xsl:value-of select="substring-after(substring-after($regexp-in,']{'),'}')"/></xsl:when>
				<xsl:when test="contains($regexp-in,']')"><xsl:value-of select="substring-after($regexp-in,']')"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="substring-after($regexp-in,$regexp)"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable -->
		<!-- xsl:message>regexp-in [<xsl:value-of select="$regexp-in"/>]</xsl:message>
		<xsl:message>regexp [<xsl:value-of select="$regexp"/>]</xsl:message>
		<xsl:message>more [<xsl:value-of select="$mask2"/>]</xsl:message -->
		<!-- xsl:choose>
		 	<xsl:when test="string-length($regexp-in) &lt; 1"></xsl:when>
			<xsl:when test="$regexp-in=$UK-postcode"><xsl:value-of select="$UK-postcode-CAM"/></xsl:when>
			<xsl:when test="$regexp-in=$UK-postcode-1"><xsl:value-of select="$UK-postcode-CAM"/></xsl:when>
			<xsl:when test="$regexp-in=$UK-postcode-BFPO"><xsl:value-of select="$UK-postcode-BFPO-CAM"/></xsl:when>
			<xsl:when test="$regexp-in=$PAON-postcode"><xsl:value-of select="$ANY-postcode-CAM"/></xsl:when>
			<xsl:when test="contains($regexp-in,$SAON-postcode)"><xsl:value-of select="$ANY-postcode-CAM"/></xsl:when -->
			<!-- this will break CAM valid XML so skip it -->
			<!-- xsl:when test="contains($regexpCAM,')')"></xsl:when>
			<xsl:when test="$regexpCAM=')'"></xsl:when -->
		    	<!-- xsl:when test="contains($regexp,'[')">
		    		<xsl:variable name="literal-A" select="replace(replace(substring-before($regexp,'['),'\(',' '),'\)',' ')"></xsl:variable>
				<xsl:variable name="charExp" select="substring-before(substring-after($regexp,'['),']')"/>
		    		<xsl:variable name="occursExp" select="substring-before(substring-after($regexp,concat($charExp,']{')),'}')"></xsl:variable>
		    		<xsl:variable name="more" select="if(string-length($occursExp) = 0) then substring-after($regexp,concat($charExp,']')) else substring-after($regexp,concat('{',$occursExp,'}'))"/>
		    		<xsl:variable name="literal-B" select="if ($more=')') then '' else substring-after($regexp,$more)"/>
		    		<xsl:value-of select="w2e:outputMask($charExp,$occursExp,$mask2,$literal-A,$literal-B)"/>
		    	</xsl:when -->
		    	<!-- xsl:otherwise><xsl:value-of select="$regexpCAM"/></xsl:otherwise>
		</xsl:choose -->
			<xsl:value-of select="if(contains($regexpCAM,$GDSC-CAM)) then $GDSC-Mask else $regexpCAM"/>
	       </xsl:when>
	<xsl:otherwise/>
	</xsl:choose>
		
	</xsl:function>
	
	<xsl:function name="w2e:outputMask"><!-- This does not get called anymore -->
		<xsl:param name="charExp"/>
		<xsl:param name="occursExp"/>
		<xsl:param name="more"/>
		<xsl:param name="literal-before"/>
		<xsl:param name="literal-after"/>		
		<xsl:variable name="blank"> </xsl:variable>		
		<xsl:variable name="occurs">
			<xsl:choose>
				<xsl:when test="string-length($occursExp) = 0"/>
				<xsl:when test="contains($occursExp,',')"><xsl:value-of select="substring-after($occursExp,',')"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$occursExp"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="prefixStrTemp"><xsl:if test="string-length($literal-before) &gt; 0 and(not($literal-before=' '))"><xsl:value-of select="concat('''',replace($literal-before,'\(',' '),'''')"/></xsl:if></xsl:variable>
		<xsl:variable name="prefixStr" select="replace(replace($prefixStrTemp,''' ',''''),' ''','''')"/>
		<!-- xsl:message>[<xsl:value-of select="$literal-before"/>][<xsl:value-of select="concat($charExp,':',$occursExp)"/>][<xsl:value-of select="$more"/>][<xsl:value-of select="$literal-after"/>]</xsl:message  -->
		<xsl:variable name="numOccurs">
			<xsl:choose>
				<xsl:when test="contains($occurs,'0')"><!-- If multiple of ten then adjust repeat count by 5 and into two parts with @@ place holder for mask char -->
					<xsl:value-of select="concat(xsd:string(xsd:integer($occurs) - 5),'@@5')"/>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$occurs"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
		<xsl:when test="not(contains($occursExp,'0,'))">
			<xsl:choose>
				<xsl:when test="$charExp='0-9'"><xsl:value-of select="concat($prefixStr,'0',replace($numOccurs,'@@','0'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='A-Z'"><xsl:value-of select="concat($prefixStr,'U',replace($occurs,'@@','U'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z'"><xsl:value-of select="concat($prefixStr,'L',replace($occurs,'@@','L'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z,0-9'"><xsl:value-of select="concat($prefixStr,'X',replace($occurs,'@@','X'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z0-9'"><xsl:value-of select="concat($prefixStr,'X',replace($occurs,'@@','X'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="contains($charExp,'A-Z') and(contains($charExp,'-9')) and (string-length($charExp) &lt; 8)"><xsl:value-of select="concat($prefixStr,'A',replace($occurs,'@@','A'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z,A-Z,0-9'"><xsl:value-of select="concat($prefixStr,'X',replace($occurs,'@@','X'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-zA-Z0-9'"><xsl:value-of select="concat($prefixStr,'X',replace($occurs,'@@','X'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='A-H,J-N,P-Z'"><xsl:value-of select="concat($prefixStr,'U',replace($occurs,'@@','U'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='0-9,A-H,J-N,P-Z'"><xsl:value-of select="concat($prefixStr,'A',replace($occurs,'@@','A'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='0-9A-HJ-NP-Z'"><xsl:value-of select="concat($prefixStr,'A',replace($occurs,'@@','A'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat($prefixStr,'X',replace($occurs,'@@','X'),w2e:regexp-2-mask($more))"/></xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:when test="contains($occursExp,'0,')">
			<xsl:choose>
				<xsl:when test="$charExp='0-9'"><xsl:value-of select="concat($prefixStr,'#',replace($occurs,'@@','#'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='A-Z'"><xsl:value-of select="concat($prefixStr,'^',replace($occurs,'@@','^'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z'"><xsl:value-of select="concat($prefixStr,'_',replace($occurs,'@@','_'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z,0-9'"><xsl:value-of select="concat($prefixStr,'a',replace($occurs,'@@','a'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z0-9'"><xsl:value-of select="concat($prefixStr,'a',replace($occurs,'@@','a'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-z,A-Z,0-9'"><xsl:value-of select="concat($prefixStr,'?',replace($occurs,'@@','?'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:when test="$charExp='a-zA-Z0-9'"><xsl:value-of select="concat($prefixStr,'?',replace($occurs,'@@','?'),w2e:regexp-2-mask($more))"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat($prefixStr,'?',replace($occurs,'@@','?'),w2e:regexp-2-mask($more))"/></xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise/>
		</xsl:choose>
	</xsl:function>
	
	<xsl:function name="w2e:nodeParentMatch">
		<xsl:param name="node"/>
		<xsl:param name="p1"/>
		<xsl:param name="p2"/>
		<xsl:param name="p3"/>
		<xsl:param name="p4"/>
		<xsl:param name="p5"/>
		<xsl:variable name="pathComponents" select="concat($p1,'|',$p2,'|',$p3,'|',$p4,'|',$p5,'|')"/>
		<xsl:variable name="nodePaths"><xsl:for-each select="tokenize($pathComponents,'\|')"><xsl:value-of select="if(. = 'xsd:choice') then '' else concat(.,'|')"/></xsl:for-each></xsl:variable>
		<xsl:variable name="nodePos"><xsl:for-each select="tokenize($nodePaths,'\|')"><xsl:if test=". = $node"><xsl:value-of select="position()"/></xsl:if></xsl:for-each></xsl:variable>
		<!-- xsl:message> - <xsl:value-of select="$pathComponents"/> - <xsl:value-of select="$nodePos"/> - <xsl:value-of select="$nodePaths"/></xsl:message -->
		<xsl:for-each select="tokenize($nodePaths,'\|')"><xsl:if test="position() = $nodePos + 1"><xsl:value-of select="."/></xsl:if></xsl:for-each>
		<!-- xsl:choose>
			<xsl:when test="$node = $p1"><xsl:value-of select="if($p2 = 'xsd:choice') then $p3 else $p2"/></xsl:when>
			<xsl:when test="$node = $p2"><xsl:value-of select="if($p3 = 'xsd:choice') then $p4 else $p3"/></xsl:when>
			<xsl:when test="$node = $p3"><xsl:value-of select="if($p4 = 'xsd:choice') then $p5 else $p4"/></xsl:when>
			<xsl:when test="$node = $p4"><xsl:value-of select="if($p5 = 'xsd:choice') then $p3 else $p5"/></xsl:when>
			<xsl:otherwise/>
		</xsl:choose -->
	</xsl:function>
	
</xsl:stylesheet>